#ephy-status-frame {
  border-style: solid;
  border-width: 1px;
  padding: 4px;
}

#ephy-tab-close-button {
  -GtkButton-default-border: 0;
  -GtkButton-default-outside-border: 0;
  -GtkButton-inner-border: 0;
  -GtkWidget-focus-padding: 0;
  -GtkWidget-focus-line-width: 0;
  margin: 0;
  padding: 0;
}

.primary-toolbar.toolbar {
    padding-left: 0;
    padding-right: 0;
}

#ephy-progress-bar {
  -GtkProgressBar-xspacing: 0;
  -GtkProgressBar-yspacing: 3px;
  -GtkProgressBar-min-horizontal-bar-height: 3px;
  padding: 0;
}

.progressbar#ephy-progress-bar {
  border-style: none;
  background-color: @theme_selected_bg_color;
  background-image: none;
  border-radius: 0;
  -adwaita-progressbar-pattern: none;
}

GtkProgressBar#ephy-progress-bar.trough {
  padding: 0;
  border-image: none;
  border-style: none;
  border-width: 0;
  background-image: none;
  background-color: transparent;
  border-radius: 0;
}

#ephy-overlay {
  background-color: rgba(0,0,0,0);
}
